import matplotlib.pyplot as plt
import pandas as pd
import matplotlib.dates as mdates
from pandas.plotting import register_matplotlib_converters
register_matplotlib_converters()
import matplotlib.ticker as tick

#Extraction des données depuis le fichier .csv :
soleil=pd.read_csv('SoleilLune2022.csv',
sep=';', 
usecols=[1, 2, 4, 6], 
names=['date', 'lever', 'midi', 'coucher'], 
parse_dates=['date'], 
skiprows=1, 
nrows=365, 
)
#Extraction des listes de données pour les graphes :
soleil=soleil.set_index('date')
dates=soleil.index
lever=soleil['lever']
coucher=soleil['coucher']
midi=soleil['midi']
EDT=midi-12#calcul de l'équation du temps

plt.rcParams["font.family"] = "Ubuntu"
plt.rcParams["font.size"] = 9

def HHmm(heure, lever):#écrit en base 60
    H=int(heure)
    min=round((heure-H)*60)
    if min<10:
        min="0"+str(min)
    return str(H)+'h'+str(min)[:-2]

def pointMinimum(liste, datemin, datemax):
    periode=liste.loc[datemin:datemax]
    Tmin=periode.idxmin(periode)
    Hmin=periode.values.min()
    ax=plt.gca()
    ax.scatter(Tmin, Hmin, c='white', facecolor='w',
    edgecolor='k', zorder=2)

def pointMaximum(liste, datemin, datemax):
    periode=liste.loc[datemin:datemax]
    Tmax=periode.idxmax(periode)
    Hmax=periode.values.max()
    ax=plt.gca()
    ax.scatter(Tmax, Hmax, c='white', facecolor='w',
    edgecolor='k', zorder=2)

#Paramètres du graphique :
fig=plt.figure(figsize=(14, 11), 
tight_layout=True
)
plt.suptitle("Heures extrêmes de lever et de coucher du Soleil - équation du temps", fontsize=16, y=0.99)

#Graphe 1---------------------------------------------------
ax1=plt.subplot(331)
ax1.set_title("Coucher du Soleil en 2022")
ax1.grid()
ax1.plot(dates, coucher, label='temps moyen', c='C0')
ax1.plot(dates, coucher-EDT, label='temps vrai', c='C2', ls='--')
#ax1.plot(dates, midi, label='midi moyen', c='yellow')

ax1.legend()

locator1 = mdates.AutoDateLocator(minticks=1, maxticks=6, interval_multiples=True)
formatter = mdates.ConciseDateFormatter(locator1)
ax1.xaxis.set_major_locator(locator1)
ax1.xaxis.set_major_formatter(formatter)
ax1.set_xlim('2022-01-01', '2022-12-31')

#Graphe 2---------------------------------------------------
ax2=plt.subplot(334)
ax2.set_title("Lever du Soleil en 2022")
ax2.grid()
ax2.plot(dates, lever, label='temps moyen', c='C1')
ax2.plot(dates, lever-EDT, label='temps vrai', c='C3', ls='--')
ax2.legend()
ax2.xaxis.set_major_locator(locator1)
ax2.xaxis.set_major_formatter(formatter)
ax2.set_xlim('2022-01-01', '2022-12-31')

#Graphe 3---------------------------------------------------
ax3=plt.subplot(337)
ax3.set_title("Équation du temps")
ax3.grid()

ax3.plot(dates, EDT, c='navy')
ax3.xaxis.set_major_locator(locator1)
ax3.xaxis.set_major_formatter(formatter)
ax3.set_xlim('2022-01-01', '2022-12-31')

def mmss(min, EDT):#écrit en base 60
    M=round(min*60)
#    sec=round((M-int(M))*60, 2)
    return str(M)[:-2]+' min'#+str(sec)[:-2]
ax3.yaxis.set_major_formatter(tick.FuncFormatter(mmss))

#Graphe 4---------------------------------------------------
ax4=plt.subplot(332)
ax4.set_title("Coucher au solstice d'été")
ax4.plot(dates, coucher, label='temps moyen', c='C0')
ax4.plot(dates, coucher-EDT, label='temps vrai', c='C2', ls='--')
ax4.grid()
ax4.yaxis.set_major_formatter(tick.FuncFormatter(HHmm))
locator2 = mdates.AutoDateLocator(minticks=15, maxticks=15, interval_multiples=True)
ax4.xaxis.set_major_locator(locator2)
ax4.xaxis.set_major_formatter(formatter)
ax4.set_xlim('2022-06-11', '2022-06-29')
ax4.set_ylim(20, 20.17)
ax4.legend()

pointMaximum(coucher, '2022-06-10', '2022-06-30')
pointMaximum(coucher-EDT, '2022-06-10', '2022-06-30')

#Graphe 5---------------------------------------------------
ax5=plt.subplot(335)
ax5.set_title("Lever au solstice d'été")
ax5.plot(dates, lever, label='temps moyen', c='C1')
ax5.plot(dates, lever-EDT, label='temps vrai', c='C3', ls='--')
ax5.grid()
ax5.yaxis.set_major_formatter(tick.FuncFormatter(HHmm))
locator2 = mdates.AutoDateLocator(minticks=15, maxticks=15, interval_multiples=True)
ax5.xaxis.set_major_locator(locator2)
ax5.xaxis.set_major_formatter(formatter)
ax5.set_xlim('2022-06-11', '2022-06-29')
ax5.set_ylim(3.85, 4.01)
ax5.legend()
pointMinimum(lever, '2022-06-10', '2022-06-30')
pointMinimum(lever-EDT, '2022-06-10', '2022-06-30')


#Graphe 6---------------------------------------------------
ax6=plt.subplot(333)
ax6.set_title("Coucher au solstice d'hiver")
ax6.plot(dates, coucher, label='temps moyen', c='C0')
ax6.plot(dates, coucher-EDT, label='temps vrai', c='C2', ls='--')
ax6.grid()
ax6.yaxis.set_major_formatter(tick.FuncFormatter(HHmm))
locator2 = mdates.AutoDateLocator(minticks=15, maxticks=15, interval_multiples=True)
ax6.xaxis.set_major_locator(locator2)
ax6.xaxis.set_major_formatter(formatter)
ax6.set_xlim('2022-12-05', '2022-12-31')
ax6.set_ylim(15.9, 16.2)
ax6.legend()

pointMinimum(coucher, '2022-12-10', '2022-12-31')
pointMinimum(coucher-EDT, '2022-12-10', '2022-12-31')

#Graphe 7---------------------------------------------------
ax7=plt.subplot(336)
ax7.set_title("Lever au solstice d'hiver")
ax7.plot(dates, lever, label='temps moyen', c='C1')
ax7.plot(dates, lever-EDT, label='temps vrai', c='C3', ls='--')
ax7.grid()
ax7.yaxis.set_major_formatter(tick.FuncFormatter(HHmm))
locator2 = mdates.AutoDateLocator(minticks=15, maxticks=15, interval_multiples=True)
ax7.xaxis.set_major_locator(locator2)
ax7.xaxis.set_major_formatter(formatter)
ax7.set_xlim('2022-12-05', '2022-12-31')
ax7.set_ylim(7.85, 8.01)
ax7.legend()

pointMaximum(lever, '2022-12-10', '2022-12-31')
pointMaximum(lever-EDT, '2022-12-10', '2022-12-31')

#Encadré 1
ax8=plt.subplot(338)
ax8.axis('off')
plt.rcParams.update({"font.family": "sans-serif"})
plt.text(0, 1,
r"\normalsize"
r"\begin{minipage}{9.50cm}"
r"\textbf{Solstice d'été : }"
r"c'est généralement le 21 juin que le jour dure le plus longtemps. "
r"Cependant, du fait de l'équation du temps (env. +2 min), c'est le 25 juin "
r"que le Soleil se couche le plus tard.\\\\"
r"De même, l'heure minimale de lever du Soleil à cette période "
r"se produit vers le 16 juin."
r"\end{minipage}"
, transform=ax8.transAxes, usetex=True, va='top')

#Encadré 2
ax9=plt.subplot(339)
ax9.axis('off')
plt.rcParams.update({"font.family": "sans-serif"})
plt.text(0, 1,
r"\normalsize"
r"\begin{minipage}{9.50cm}"
r"\textbf{Solstice d'hiver : }"
r"l'équation du temps est négative et s'annule le 25 décembre. "
r"L'heure de coucher du Soleil est minimale le 12 décembre. "
r"Au-delà de cette date, le Soleil se couche plus tard, "
r"bien que les journées continuent à raccourcir (jusqu'au 21 décembre, en général).\\\\"
r"L'heure de lever de Soleil continue à augmenter jusqu'au 31 décembre.\\\\\\"
r"Les calculs d’éphémérides de position des corps du Système solaire"
r" ont été réalisés par le service de calcul des éphémérides de l’IMCCE"
r" à travers son portail Système solaire (https://ssp.imcce.fr)"
r"\end{minipage}"
, transform=ax9.transAxes, usetex=True, va='top')

fig.savefig("LeverCoucherSoleil.png", dpi=400)
#plt.show()
